--    cr2hdr developped by a1ex
--    http://www.magiclantern.fm/forum/index.php?topic=7139.0
--    Plugin developped for Magic Lantern
--    Add "cr2hdr" to export
--
--    Metadata added by akry
--
--    Copyright (C) <2014>  <kichetof kichetof@gmail.com>
--
--    This program is free software; you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation; either version 2 of the License, or
--    (at your option) any later version.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License along
--    with this program; if not, write to the Free Software Foundation, Inc.,
--    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

local LrApplication = import 'LrApplication'
local LrColor = import 'LrColor'
local LrDialogs = import 'LrDialogs'
local LrErrors = import 'LrErrors'
local LrExportSession = import 'LrExportSession'
local LrFileUtils = import 'LrFileUtils'
local LrFunctionContext = import 'LrFunctionContext'
local LrHttp = import 'LrHttp'
local LrLogger = import 'LrLogger'
local LrPathUtils = import 'LrPathUtils'
local LrProgressScope = import 'LrProgressScope'
local LrTasks = import 'LrTasks'
local LrView = import "LrView"

local MLProcess = {}

-- Check for Mac dependencies
function MLProcess.checkDependencies()
    return "exec which dcraw exiftool"
end

-- Fill empty table for summary message
function MLProcess.checkTab (tab, err)
    local msg = err ~= 1 and LOC "$$$/ML/Publish/Finish/NoImage=No image" or LOC "$$$/ML/Publish/Finish/NoError=No error"
    return #tab == 0 and table.insert(tab, { ["title"] = msg, ["value"] = "" }) or tab
end

function MLProcess.getSubfolderPath(path, subfolder)
    local filename = LrPathUtils.leafName(path)
    local pathNofilename = LrPathUtils.parent(path)
    local newPath = LrPathUtils.child(pathNofilename, subfolder)
    LrFileUtils.createAllDirectories(newPath)
    return LrPathUtils.child(newPath, filename)
end

function MLProcess.getRenamePath(path, suffix)
    local pathNoExt = LrPathUtils.removeExtension(path)
    local ext = LrPathUtils.extension(path)
    local finalPath = string.format("%s-%s", pathNoExt, suffix)
    return LrPathUtils.addExtension(finalPath, ext)
end

function MLProcess.moveToSubFolder(destPath, subFolder)
    local finalSubfolderPath = MLProcess.getSubfolderPath(destPath, subFolder)
    mlLog:debugf("subfolder: %s", finalSubfolderPath)
    if not LrFileUtils.move(destPath, finalSubfolderPath) then
        return false, destPath
    else
        return true, finalSubfolderPath
    end
end

function MLProcess.generateArgs(exportSettings)

    local args = string.format('%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s',
        exportSettings.shortcut_fast == true and " --fast" or "",
        exportSettings.interp_method == 0 and " --amaze-edge" or " --mean23",
        exportSettings.chroma_smooth_method == 0 and " --no-cs" or string.format(" --cs%dx%d", exportSettings.chroma_smooth_method, exportSettings.chroma_smooth_method),
        exportSettings.fix_bad_pixels == 0 and " --no-bad-pix" or ((exportSettings.fix_bad_pixels == 1) and " --bad-pix" or " --really-bad-pix"),
        exportSettings.debug_bad_pixels == true and " --black-bad-pix" or "",
        exportSettings.use_fullres == 0 and " --no-fullres" or " --fullres",
        exportSettings.use_alias_map == 0 and " --no-alias-map" or " --alias-map",
        exportSettings.use_stripe_fix == 0 and " --no-stripe-fix" or " --stripe-fix",
        exportSettings.compress == 1 and " --compress" or ((exportSettings.compress == 2) and "--compress-lossy" or ""),
        exportSettings.debug_blend == true and " --debug-blend" or "",
        exportSettings.debug_black == true and " --debug-black" or "",
        exportSettings.debug_amaze == true and " --debug-amaze" or "",
        exportSettings.debug_edge == true and " --debug-edge" or "",
        exportSettings.debug_alias == true and " --debug-alias" or "",
        exportSettings.plot_iso_curve == true and " --iso-curve" or "",
        exportSettings.plot_mix_curve == true and " --mix-curve" or "",
        exportSettings.plot_fullres_curve == true and " --fullres-curve" or ""
    )

    if exportSettings.cr2hdr == "cr2hdr20bit" then
        args = string.format('%s%s%s%s%s%s%s%s%s',
            args,
            exportSettings.soft_film > 0 and string.format(" --soft-film=%.1f", exportSettings.soft_film) or "",
            exportSettings.gray_wb == 0 and " --wb=graymax" or exportSettings.gray_wb == 1 and " --wb=graymed" or exportSettings.gray_wb == 2 and " --wb=exif" or "",
            exportSettings.same_levels == true and " --same-levels" or "",
            exportSettings.skip_existing == true and " --skip-existing" or "",
            exportSettings.embed_original == true and " --embed-original-copy" or "",
            exportSettings.debug_rggb == true and " --debug-rggb" or "",
            exportSettings.debug_bddb == true and " --debug-bddb" or "",
            exportSettings.debug_wb == true and " --debug-wb" or ""
        )
    end

    mlLog:debugf("arguments : %s", args)
    return args
end

-- Return command line for cr2hdr
function MLProcess.generateCmd(exportSettings, pluginPath, filePath, output, args)

    local exec = exportSettings.cr2hdr
    local gt = " > "
    local cmd = WIN_ENV and 'start /D "%s\\bin" /B /WAIT /BELOWNORMAL %s.exe%s "%s"' or 'exec "%s/bin/%s"%s %s'
    local log = '%s%s"%s"'
    local escape = WIN_ENV and '"%s"' or '%s'

    return string.format(escape,
        string.format(log,
            string.format(cmd,
                pluginPath,
                exec,
                args,
                filePath
            ),
            gt,
            output)
    )
end

-- remove trailing and leading spaces
function MLProcess.trim(s)
    return (s:gsub("^%s*(.-)%s*$", "%1"))
end

function MLProcess.split(myString, inSplitPattern, outResults)

    if not outResults then
        outResults = { }
    end

    local theStart = 1
    local theSplitStart, theSplitEnd = string.find(myString, inSplitPattern, theStart)

    while theSplitStart do
        table.insert( outResults, string.sub( myString, theStart, theSplitStart-1 ) )
        theStart = theSplitEnd + 1
        theSplitStart, theSplitEnd = string.find( myString, inSplitPattern, theStart )
    end

    table.insert(outResults, string.sub(myString, theStart))
    return outResults
end

function MLProcess.checkDngExtSuffix(photo, suffix)
    if photo:getRawMetadata("fileFormat") == "DNG" and suffix == "" then
        return true
    else
        return false
    end
end

function MLProcess.checkKeyword(photo, keyword)
    local keywords = photo:getRawMetadata("keywords")
    for i, key in ipairs(keywords) do
        if key:getName() == keyword then
            return true
        end
    end
    return false
end

function MLProcess.importDualIsoConverted(catalog, photo, originalFilePath, finalPathDNG, finalPathTXT, exportSettings, args)
    local originalCollection = photo:getContainedCollections()
    local importDualiso = catalog:withWriteAccessDo('Import from ML DualISO', function(context)
            local photoImported = catalog:addPhoto(finalPathDNG, photo, 'above')

            if exportSettings.label ~= "none" then
                photoImported:setRawMetadata('label', exportSettings.label)
            end

            if exportSettings.keywords ~= '' then
                local keywords = MLProcess.split(exportSettings.keywords, ",")
                local keyword
                for i, key in ipairs (keywords) do
                    key = MLProcess.trim(key)
                    if key ~= "" then
                        keyword = catalog:createKeyword(key, nil, true, nil, true)
                        photoImported:addKeyword(keyword)
                    end
                end
            end

            -- sync keywords from original picture
            if exportSettings.sync_keywords and MLProcess.checkDngExtSuffix(photo, exportSettings.suffix) == false then
                local keywords = photo:getRawMetadata("keywords")
                for i, key in ipairs(keywords) do
                    if key:getName() ~= "Dual-ISO raw" then
                        photoImported:addKeyword(key)
                    end
                end
            end

            -- FUTURE if possible ? sync importation preset

            if LrFileUtils.exists(finalPathTXT) then
                local metadata = MLProcess.parseLogFileToArray(finalPathTXT)
                MLProcess.fillMetadata(photoImported, metadata[LrPathUtils.leafName(originalFilePath)])
            end

            local cr2hdrExec = exportSettings.cr2hdr == "cr2hdr20bit" and "cr2hdr-20bit" or "cr2hdr"
            photoImported:setPropertyForPlugin (_PLUGIN, 'cr2hdrExec', cr2hdrExec)
            photoImported:setPropertyForPlugin (_PLUGIN, 'cr2hdrArgs', args)

            if exportSettings.addToCollection and next(originalCollection) ~= nil then
                for k, v in pairs(originalCollection) do
                    v:addPhotos({ photoImported })
                end
            end

            if exportSettings.saveOutput == false and exportSettings.same_levels == false then
                LrFileUtils.moveToTrash(finalPathTXT)
            end

            if MLProcess.checkDngExtSuffix(photo, exportSettings.suffix) == false then
                if exportSettings.flag == true then
                    photo:setRawMetadata('pickStatus', -1)
                end
                -- Add a keyword to original picture to simplify identification Dual-ISO raw / no Dual-ISO
                keyword = catalog:createKeyword("Dual-ISO raw", nil, true, nil, true)
                photo:addKeyword(keyword)
            end
        end,
        {timeout = 30}
    )

    if importDualiso ~= "executed" then
        return false
    else
        return true
    end
end

function MLProcess.dualIsoProcess(catalog, photo, originalFilePath, finalPathDNG, finalPathTXT, summary, exportSettings, args)
    if LrFileUtils.exists(finalPathDNG) ~= false then
        if exportSettings.suffix ~= "" then
            local finalPath = MLProcess.getRenamePath(finalPathDNG, exportSettings.suffix)
            local finalPathLog = MLProcess.getRenamePath(finalPathTXT, exportSettings.suffix)
            local moveLogFile = LrFileUtils.move(finalPathTXT, finalPathLog) -- don't check if move success, bypass problem with --same-levels
            finalPathTXT = finalPathLog
            if LrFileUtils.move(finalPathDNG, finalPath) == false then
                summary.errorMsg[#summary.errorMsg+1] = { ["title"] = LOC("$$$/ML/Import/ErrorSuffix=Unable to add suffix ^1 to file ^2", exportSettings.suffix, LrPathUtils.leafName(finalPathDNG)), ["value"] = "" }
            else
                finalPathDNG = finalPath
            end
        end

        if MLProcess.checkDngExtSuffix(photo, exportSettings.suffix) == true then
            local finalPath = LrFileUtils.chooseUniqueFileName(finalPathDNG)
            local finalPathLog = LrFileUtils.chooseUniqueFileName(finalPathTXT)
            local moveLogFile = LrFileUtils.move(finalPathTXT, finalPathLog) -- don't check if move success, bypass problem with --same-levels
            finalPathTXT = finalPathLog
            if LrFileUtils.move(finalPathDNG, finalPath) == false then
                summary.errorMsg[#summary.errorMsg+1] = { ["title"] = LOC("$$$/ML/Import/ErrorSuffix=Unable to add suffix ^1 to file ^2", exportSettings.suffix, LrPathUtils.leafName(finalPathDNG)), ["value"] = "" }
            else
                finalPathDNG = finalPath
            end
        end

        -- after cr2hdr, move file to subfolder
        if exportSettings.subfolder ~= "" then
            finalPathDNGError, finalPathDNG = MLProcess.moveToSubFolder(finalPathDNG, exportSettings.subfolder)
            finalPathTXTError, finalPathTXT = MLProcess.moveToSubFolder(finalPathTXT, exportSettings.subfolder)

            if finalPathDNGError == false then
                summary.errorMsg[#summary.errorMsg+1] = { ["title"] = LOC("$$$/ML/Import/ErrorSubfolder=Unable to move ^1 to subfolder ^2", LrPathUtils.leafName(finalPathDNG), exportSettings.subfolder), ["value"] = "" }
            end
        end

        local import = MLProcess.importDualIsoConverted(catalog, photo, originalFilePath, finalPathDNG, finalPathTXT, exportSettings, args)
        if import == true then
            summary.success[#summary.success+1] = { ["title"] = LrPathUtils.leafName(originalFilePath), ["value"] = "" }
        else
            summary.errorMsg[#summary.errorMsg+1] = { ["title"] = LOC "$$$/ML/Import/Error=Unable to import the Dual ISO file!^n^nPlease manually synchronise your folder", ["value"] = "" }
        end
    else
        local importDualiso = catalog:withWriteAccessDo('Import from ML DualISO', function(context)
                -- Add a keyword to original picture to simplify identification Dual-ISO raw / no Dual-ISO
                keyword = catalog:createKeyword("no Dual-ISO", nil, true, nil, true)
                photo:addKeyword(keyword)
            end,
            {timeout = 30}
        )
        LrFileUtils.moveToTrash(finalPathTXT)
        summary.nodual[#summary.nodual+1] = { ["title"] = LrPathUtils.leafName(originalFilePath), ["value"] = "" }
    end
end

-- parse logfile to read info
function MLProcess.parseLogFileToArray(filename)

    local logFileOpen = assert(io.open(filename, 'r'))
    local logfile = logFileOpen:read("*all")
    logFileOpen:close()
    info = {}
    info_index = {}
    masterKey = "none"
    local version = ""

    -- parse cr2hdr output for "xxx : yyy" and store keys/values in temp_t
    for k, v in string.gmatch(logfile, '%s*([%w%s-]+)%s*:%s*([%S ]+)%s*') do
        k = MLProcess.trim(k)
        -- keep input file to store in master keys
        if k == "Input file" then
            masterKey = LrPathUtils.leafName(MLProcess.trim(v))
        end

        if k == "Last update" then
            version = MLProcess.trim(v)
        end

        if info[masterKey] == nil then
            info[masterKey] = {}
            info_index[masterKey] = {}
        end

        -- there are duplicating keys in output (for instance, "Dynamic range")
        if info[masterKey][k] ~= nil then
            local t = k
            info_index[masterKey][t] = info_index[masterKey][t] + 1
            k = k .. info_index[masterKey][t]
        else
            info_index[masterKey][k] = 1
        end

        info[masterKey][k] = MLProcess.trim(v)

        if version ~= "" and info[masterKey]['Last update'] == nil then
            info[masterKey]["Last update"] = version
        end
    end

    return info
end

-- fill metadata
function MLProcess.fillMetadata(photo, meta)

    -- exit function if metadata array is empty
    if meta == nil then
        do return end
    end

    local metaString1
    local metaString2

    if meta['ISO difference'] ~= nil then
        metaString1 = meta['ISO difference']:match("(%S+) EV")
        local isoFirst = photo:getRawMetadata("isoSpeedRating")
        if isoFirst ~= nil then
            local isoRange = string.format("%s - %s", isoFirst, 2^math.floor(metaString1+0.5)*isoFirst)
            photo:setPropertyForPlugin (_PLUGIN, 'cr2hdrISORange', isoRange)
        end
        metaString1 = string.format("%s EV", metaString1)
        photo:setPropertyForPlugin (_PLUGIN, 'ISOdifference', metaString1)
    end

    if meta['Dynamic range2'] ~= nil then
        metaString1 = meta['Dynamic range2']:match("(%S+ EV)")
        photo:setPropertyForPlugin (_PLUGIN, 'DynamicRange', metaString1)
    end

    if meta['Dynamic range'] ~= nil then
        metaString1, metaString2 = meta['Dynamic range']:match("(%S+) %(%+%) (%S+)")
        photo:setPropertyForPlugin (_PLUGIN, 'DynamicRangeOrig1', metaString1 .. ' EV')
        photo:setPropertyForPlugin (_PLUGIN, 'DynamicRangeOrig2', metaString2 .. ' EV')
    end

    if meta['Semi-overexposed'] ~= nil then
        metaString1 = meta['Semi-overexposed']:match("(%S+%%)")
        photo:setPropertyForPlugin (_PLUGIN, 'SemiOverexposed', metaString1)
    end

    if meta['Deep shadows'] ~= nil then
        metaString1 = meta['Deep shadows']:match("(%S+%%)")
        photo:setPropertyForPlugin (_PLUGIN, 'DeepShadows', metaString1)
    end

    if meta['Noise level'] ~= nil then
        metaString1, metaString2 = meta['Noise level']:match("(%S+) .+, ideally (%S+)")
        photo:setPropertyForPlugin (_PLUGIN, 'NoiseLevel', metaString1)
        photo:setPropertyForPlugin (_PLUGIN, 'NoiseLevelIdeal', metaString2)
    end

    if meta['ISO overlap'] ~= nil then
        metaString1 = meta['ISO overlap']:match("(%S+ EV)")
        photo:setPropertyForPlugin (_PLUGIN, 'ISOoverlap', metaString1)
    end

    if meta['Last update'] ~= nil then
        metaString1, metaString2 = meta['Last update']:match("(%S+) on (%S+)")
        local lastUpd = string.format("%s (%s)", metaString2, metaString1)
        photo:setPropertyForPlugin(_PLUGIN, 'cr2hdrVersion', lastUpd)
    end
end

function MLProcess.process(functionContext, exportContext)

    -- var for summary message
    local summary = {
        ["success"] = {},
        ["nodual"] = {},
        ['dual'] = {},
        ['format'] = {},
        ["fail"] = {},
        ["errorMsg"] = {}
    }
    functionContext:addCleanupHandler(
        function(status, message)
            if not status then
                  LrDialogs.message("~ There has been an error ~", string.format("Please report the problem to Magic Lantern forum\n\n%s", message), "critical")
            end
        end
    )
    if MAC_ENV then
        local cmd = MLProcess.checkDependencies()
        isDependOk = LrTasks.execute(cmd)
        if isDependOk ~= 0 then
            LrDialogs.message(LOC "$$$/ML/Dependencies/ErrorTitle=~ Dependencies error ~", LOC "$$$/ML/Dpendencies/Error=This plugin (cr2hdr) need dcraw and exiftool.^n^nPlease install it before run this plugin")
            do return end
        end
    end

    local catalog = LrApplication:activeCatalog()
    local exportSession = exportContext.exportSession
    local photosToRendering = ""
    local exportSettings = exportContext.propertyTable
    local nPhotos = exportSession:countRenditions()
    local progressScope = exportContext:configureProgress {
        title = nPhotos > 1 and LOC("$$$/ML/Publish/Progress=Exporting ^1 photos to cr2hdr", nPhotos) or LOC "$$$/ML/Publish/Progress/One=Exporting one photo to cr2hdr",
    }
    local i = 0
    local firstFilePath = ""

    for photo in exportSession:photosToExport() do
        i = i + 1
        local originalFilePath = photo:getRawMetadata("path")
        local fileFormat = photo:getRawMetadata("fileFormat")
        local pluginPath = _PLUGIN.path
        local testDualIsoKeyword = MLProcess.checkKeyword(photo, "Dual-ISO")
        local testNoDualIsoKeyword = MLProcess.checkKeyword(photo, "no Dual-ISO")

        if photo:checkPhotoAvailability() and (fileFormat == "RAW" or fileFormat == "DNG") and testDualIsoKeyword == false and testNoDualIsoKeyword == false then
            local finalPathDNG = LrPathUtils.replaceExtension(originalFilePath, "DNG")
            local finalPathTXT = LrPathUtils.replaceExtension(originalFilePath, "TXT")
            mlLog:debugf("Path: %s", originalFilePath)
            mlLog:debugf("Path DNG: %s", finalPathDNG)
            mlLog:debugf("Path TXT: %s", finalPathTXT)

            if i == 1 then
                firstFilePath = LrPathUtils.removeExtension(finalPathTXT)
                mlLog:debugf("first: %s", firstFilePath)
            end

            -- use for --same-levels
            photosToRendering = string.format("%s \"%s\"", photosToRendering, originalFilePath)

            if exportSettings.same_levels == false then
                progressScope:setPortionComplete((i - 1) / nPhotos)
                local args = MLProcess.generateArgs(exportSettings)
                local command = MLProcess.generateCmd(exportSettings, pluginPath, string.format("\"%s\"", originalFilePath), finalPathTXT, args)
                mlLog:debugf("command : %s", command)

                result = LrTasks.execute(command)

                if result == 0 then
                    MLProcess.dualIsoProcess(catalog, photo, originalFilePath, finalPathDNG, finalPathTXT, summary, exportSettings, args)
                else
                    summary.errorMsg[#summary.errorMsg+1] =  { ["title"] = LOC("$$$/ML/Publish/Error=Unable to export ^1 to cr2hdr! Error ^2", originalFilePath, result), ["value"] = "" }
                    break
                end
            end

            if exportSettings.cr2hdr == "cr2hdr20bit" and exportSettings.same_levels == true and i == nPhotos then
                progressScope:setPortionComplete(0)
                local lastFilePathName = LrPathUtils.leafName(photo:getRawMetadata("path"))
                local finalLogFile = string.format("%s-%s", firstFilePath, lastFilePathName)
                local finalPathTXT = LrPathUtils.replaceExtension(finalLogFile, "TXT")
                local args = MLProcess.generateArgs(exportSettings)
                local command = MLProcess.generateCmd(exportSettings, pluginPath, MLProcess.trim(photosToRendering), finalPathTXT, args)
                mlLog:debugf("last: %s", lastFilePathName)
                mlLog:debugf("logfile: %s", finalPathTXT)
                mlLog:debugf("command : %s", command)

                result = LrTasks.execute(command)

                if result == 0 then
                    for photo in exportSession:photosToExport() do
                        originalFilePath = photo:getRawMetadata("path")
                        finalPathDNG = LrPathUtils.replaceExtension(originalFilePath, "DNG")
                            MLProcess.dualIsoProcess(catalog, photo, originalFilePath, finalPathDNG, finalPathTXT, summary, exportSettings, args)
                    end

                    if exportSettings.saveOutput == false then
                        if exportSettings.suffix ~= "" then
                            finalPathTXT = MLProcess.getRenamePath(finalPathTXT, exportSettings.suffix)
                        end
                        if MLProcess.checkDngExtSuffix(photo, exportSettings.suffix) == true then
                            finalPathTXT = LrFileUtils.chooseUniqueFileName(finalPathTXT)
                        end
                        if exportSettings.subfolder ~= "" then
                            finalPathTXTError, finalPathTXT = MLProcess.moveToSubFolder(finalPathTXT, exportSettings.subfolder)
                        end

                        LrFileUtils.moveToTrash(finalPathTXT)
                    end
                end
            end
        else
            if photo:checkPhotoAvailability() == false then
                summary.errorMsg[#summary.errorMsg+1] =  { ["title"] = LOC("$$$/ML/Publish/PhotoAvailabilityError=Photo ^1 are not present on the disk", LrPathUtils.leafName(originalFilePath)), ["value"] = "" }
            elseif testDualIsoKeyword == true then
                summary.dual[#summary.dual+1] = { ["title"] = LrPathUtils.leafName(originalFilePath), ["value"] = "" }
            elseif testNoDualIsoKeyword == true then
                summary.nodual[#summary.nodual+1] = { ["title"] = LrPathUtils.leafName(originalFilePath), ["value"] = "" }
            else
                summary.format[#summary.format+1] = { ["title"] = LrPathUtils.leafName(originalFilePath), ["value"] = "" }
            end
        end
    end

    progressScope:done()

    -- Show final message
    mlLog:debugf("success: %d - nodual: %d - dual: %d - format: %d - fail: %d - errorMsg: %d",
        #summary.success,
        #summary.nodual,
        #summary.dual,
        #summary.format,
        #summary.fail,
        #summary.errorMsg
        )

    if exportSettings.showSummary then

        local v = LrView.osFactory()
        local totalImg = #summary.success
        local margin = 5
        local width = 600

        LrDialogs.presentModalDialog {
            title = LOC "$$$/ML/Publish/Finish=cr2hdr export finished",
            resizable = "vertically",
            cancelVerb = "< exclude >",
            contents = v:view {
                width = width,
                v:static_text {
                    width = width,
                    title = totalImg == 0 and LOC "$$$/ML/Publish/NoPhotoExported=No photo has been exported" or totalImg <= 1 and LOC("$$$/ML/Publish/NbPhotoExported=^1 photo has been exported with success", totalImg) or LOC("$$$/ML/Publish/NbPhotosExported=^1 photos have been exported with success", totalImg),
                    alignment = "center",
                    font = "<system/bold>",
                },
                v:spacer {
                    height = 20
                },
                v:tab_view {
                    value = "FinishTab1",
                    fill_horizontal = 1,
                    v:tab_view_item {
                        title = LOC("$$$/ML/Publish/Finish/Success=Converted^1", (#summary.success > 0 and " (" .. #summary.success .. ")" or "")),
                        identifier = "FinishTab1",
                        v:simple_list {
                            width = width,
                            items = MLProcess.checkTab(summary.success, 0),
                        },
                    },
                    v:tab_view_item {
                        title = LOC("$$$/ML/Publish/Finish/noDual=No DualISO^1", (#summary.nodual > 0 and " (" .. #summary.nodual .. ")" or "")),
                        identifier = "FinishTab2",
                        v:simple_list {
                            width = width,
                            items = MLProcess.checkTab(summary.nodual, 0),
                        },
                    },
                    v:tab_view_item {
                        title = LOC("$$$/ML/Publish/Finish/AlreadyConvertTitle=Already treated^1", (#summary.dual > 0 and " (" .. #summary.dual .. ")" or "")),
                        identifier = "FinishTab3",
                        v:simple_list {
                            width = width,
                            items = MLProcess.checkTab(summary.dual, 0),
                        },
                    },
                    v:tab_view_item {
                        title = LOC("$$$/ML/Publish/Finish/NotRawTitle=Wrong format^1", (#summary.format > 0 and " (" .. #summary.format .. ")" or "")),
                        identifier = "FinishTab4",
                        v:simple_list {
                            width = width,
                            items = MLProcess.checkTab(summary.format, 0),
                        },
                    },
                    v:tab_view_item {
                        title = LOC("$$$/ML/Publish/Finish/Error=Error^1", (#summary.errorMsg > 0 and " (" .. #summary.errorMsg .. ")" or "")),
                        identifier = "FinishTab5",
                        v:simple_list {
                            width = width,
                            items = MLProcess.checkTab(summary.errorMsg, 1),
                        },
                    },
                },
            }
        }

    end

end

return MLProcess
